@@ -577,8 +577,13 @@ import_cython_function(const char *module_name, const char *function_name)
         return NULL;
     cobj = PyMapping_GetItemString(capi, function_name);
     Py_DECREF(capi);
-    if (cobj == NULL)
+    if (cobj == NULL) {
+	PyErr_Clear();
+	PyErr_Format(PyExc_ValueError,
+		     "No function '%s' found in __pyx_capi__ of '%s'",
+		     function_name, module_name);
         return NULL;
+    }
     /* 2.7+ => Cython exports a PyCapsule */
     capsule_name = PyCapsule_GetName(cobj);
     if (capsule_name != NULL) {
@@ -588,6 +593,30 @@ import_cython_function(const char *module_name, const char *function_name)
     return res;
 }
 
+NUMBA_EXPORT_FUNC(PyObject *)
+_numba_import_cython_function(PyObject *self, PyObject *args)
+{
+    const char *module_name;
+    const char *function_name;
+    void *p = NULL;
+    PyObject *res;
+
+    if (!PyArg_ParseTuple(args, "ss", &module_name, &function_name)) {
+	return NULL;
+    }
+    p = import_cython_function(module_name, function_name);
+    if (p == NULL) {
+	return NULL;
+    }
+    res = PyLong_FromVoidPtr(p);
+    if (res == NULL) {
+      PyErr_SetString(PyExc_RuntimeError,
+		      "Could not convert function address to int");
+      return NULL;
+    }
+    return res;
+}
+
 /* We use separate functions for datetime64 and timedelta64, to ensure
  * proper type checking.
  */
@@ -666,49 +695,6 @@ numba_py_type(PyObject *obj) {
     return (PyObject *) Py_TYPE(obj);
 }
 
-/* Pointer-stuffing functions for tagging a Python list object with an
- * arbitrary pointer.
- * Note a similar hack is used by Python itself, since
- * "list.sort() temporarily sets allocated to -1 to detect mutations".
- */
-
-NUMBA_EXPORT_FUNC(void)
-numba_set_list_private_data(PyListObject *listobj, void *ptr)
-{
-    /* Since ptr is dynamically allocated, it is at least
-     * 4- or 8-byte-aligned, meaning we can shift it by a couple bits
-     * to the right without losing information.
-     */
-    if ((size_t) ptr & 1) {
-        /* Should never happen */
-        Py_FatalError("Numba_set_list_private_data got an unaligned pointer");
-    }
-    /* Make the pointer distinguishable by forcing it into a negative
-     * number (obj->allocated is normally positive, except when sorting
-     * where it's changed to -1).
-     */
-    listobj->allocated = - (Py_ssize_t) ((size_t) ptr >> 1);
-}
-
-NUMBA_EXPORT_FUNC(void *)
-numba_get_list_private_data(PyListObject *listobj)
-{
-    if (listobj->allocated < -1) {
-        /* A Numba pointer is stuffed in the list, return it */
-        return (void *) ((size_t) -listobj->allocated << 1);
-    }
-    return NULL;
-}
-
-NUMBA_EXPORT_FUNC(void)
-numba_reset_list_private_data(PyListObject *listobj)
-{
-    /* Pretend there is no over-allocation; this should be always correct,
-     * if not optimal.
-     */
-    if (listobj->allocated < -1)
-        listobj->allocated = PyList_GET_SIZE(listobj);
-}
 
 /*
  * Functions for tagging an arbitrary Python object with an arbitrary pointer.
@@ -848,10 +834,15 @@ numba_do_raise(PyObject *exc)
         /* Reraise */
         PyThreadState *tstate = PyThreadState_GET();
         PyObject *tb;
+#if (PY_MAJOR_VERSION >= 3) && (PY_MINOR_VERSION >= 7)
+        _PyErr_StackItem *tstate_exc = tstate->exc_info;
+#else
+        PyThreadState *tstate_exc = tstate;
+#endif
         Py_DECREF(exc);
-        type = tstate->exc_type;
-        value = tstate->exc_value;
-        tb = tstate->exc_traceback;
+        type = tstate_exc->exc_type;
+        value = tstate_exc->exc_value;
+        tb = tstate_exc->exc_traceback;
         if (type == Py_None) {
             PyErr_SetString(PyExc_RuntimeError,
                             "No active exception to reraise");
