@@ -487,12 +487,30 @@ def int_sign_impl(context, builder, sig, args):
     return impl_ret_untracked(context, builder, sig.return_type, res)
 
 
+def bool_negate_impl(context, builder, sig, args):
+    [typ] = sig.args
+    [val] = args
+    res = context.cast(builder, val, typ, sig.return_type)
+    res = builder.neg(res)
+    return impl_ret_untracked(context, builder, sig.return_type, res)
+
+
+def bool_unary_positive_impl(context, builder, sig, args):
+    [typ] = sig.args
+    [val] = args
+    res = context.cast(builder, val, typ, sig.return_type)
+    return impl_ret_untracked(context, builder, sig.return_type, res)
+
+
 lower_builtin('==', types.boolean, types.boolean)(int_eq_impl)
 lower_builtin('!=', types.boolean, types.boolean)(int_ne_impl)
 lower_builtin('<', types.boolean, types.boolean)(int_ult_impl)
 lower_builtin('<=', types.boolean, types.boolean)(int_ule_impl)
 lower_builtin('>', types.boolean, types.boolean)(int_ugt_impl)
 lower_builtin('>=', types.boolean, types.boolean)(int_uge_impl)
+lower_builtin('-', types.boolean)(bool_negate_impl)
+lower_builtin('+', types.boolean)(bool_unary_positive_impl)
+
 
 @lower_builtin('==', types.Const, types.Const)
 def const_eq_impl(context, builder, sig, args):
