@@ -327,6 +327,14 @@ class TestArrayAnalysis(TestCase):
                                asserts=[self.with_assert('A', 'B'),
                                         self.without_assert('C', 'D')])
 
+        def test_tup_arg(T):
+            T2 = T
+            return T2[0]
+
+        int_arr_typ = types.Array(types.intp, 1, 'C')
+        self._compile_and_test(test_tup_arg,
+            (types.Tuple((int_arr_typ, int_arr_typ)),), asserts=None)
+
         T = namedtuple("T", ['a','b'])
         def test_namedtuple(n):
             r = T(n, n)
@@ -334,6 +342,14 @@ class TestArrayAnalysis(TestCase):
         self._compile_and_test(test_namedtuple, (types.intp,),
                                 equivs=[self.with_equiv('r', ('n', 'n'))],)
 
+        # np.where is tricky since it returns tuple of arrays
+        def test_np_where_tup_return(A):
+            c = np.where(A)
+            return len(c[0])
+
+        self._compile_and_test(test_np_where_tup_return,
+            (types.Array(types.intp, 1, 'C'),), asserts=None)
+
         def test_shape(A):
             (m, n) = A.shape
             B = np.ones((m, n))
@@ -832,6 +848,13 @@ class TestArrayAnalysis(TestCase):
                                equivs=[self.with_equiv('a', 'c', 'e')],
                                asserts=None)
 
+class TestArrayAnalysisParallelRequired(TestCase):
+    """This is to just split out tests that need the parallel backend and
+    therefore serialised execution.
+    """
+
+    _numba_parallel_test_ = False
+
     @skip_unsupported
     def test_misc(self):
 
