@@ -7,7 +7,7 @@ import numpy as np
 
 from numba import unittest_support as unittest
 from numba.compiler import compile_isolated
-from numba import jit, types, errors
+from numba import jit, types, errors, utils
 from .support import TestCase, MemoryLeakMixin, tag
 
 
@@ -500,5 +500,79 @@ class TestMethods(TestCase):
         self.assertEqual(msg, str(raises.exception))
 
 
+class TestTupleBuild(TestCase):
+
+    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
+    def test_build_unpack(self):
+        def check(p):
+            # using eval here since Python 2 doesn't even support the syntax
+            pyfunc = eval("lambda a: (1, *a)")
+            cfunc = jit(nopython=True)(pyfunc)
+            self.assertPreciseEqual(cfunc(p), pyfunc(p))
+
+        # Homogeneous
+        check((4, 5))
+        # Heterogeneous
+        check((4, 5.5))
+
+
+    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
+    def test_build_unpack_more(self):
+        def check(p):
+            # using eval here since Python 2 doesn't even support the syntax
+            pyfunc = eval("lambda a: (1, *a, (1, 2), *a)")
+            cfunc = jit(nopython=True)(pyfunc)
+            self.assertPreciseEqual(cfunc(p), pyfunc(p))
+
+        # Homogeneous
+        check((4, 5))
+        # Heterogeneous
+        check((4, 5.5))
+
+
+    @unittest.skipIf(utils.PYVERSION < (3, 0), "needs Python 3")
+    def test_build_unpack_call(self):
+        def check(p):
+            # using eval here since Python 2 doesn't even support the syntax
+            @jit
+            def inner(*args):
+                return args
+            pyfunc = eval("lambda a: inner(1, *a)", locals())
+            cfunc = jit(nopython=True)(pyfunc)
+            self.assertPreciseEqual(cfunc(p), pyfunc(p))
+
+        # Homogeneous
+        check((4, 5))
+        # Heterogeneous
+        check((4, 5.5))
+
+    @unittest.skipIf(utils.PYVERSION < (3, 6), "needs Python 3.6+")
+    def test_build_unpack_call_more(self):
+        def check(p):
+            # using eval here since Python 2 doesn't even support the syntax
+            @jit
+            def inner(*args):
+                return args
+            pyfunc = eval("lambda a: inner(1, *a, *(1, 2), *a)", locals())
+            cfunc = jit(nopython=True)(pyfunc)
+            self.assertPreciseEqual(cfunc(p), pyfunc(p))
+
+        # Homogeneous
+        check((4, 5))
+        # Heterogeneous
+        check((4, 5.5))
+
+    def test_tuple_constructor(self):
+        def check(pyfunc, arg):
+            cfunc = jit(nopython=True)(pyfunc)
+            self.assertPreciseEqual(cfunc(arg), pyfunc(arg))
+
+        # empty
+        check(lambda _: tuple(), ())
+        # Homogeneous
+        check(lambda a: tuple(a), (4, 5))
+        # Heterogeneous
+        check(lambda a: tuple(a), (4, 5.5))
+
 if __name__ == '__main__':
     unittest.main()
